import re
from fuzzywuzzy import fuzz
from fuzzywuzzy import process

from vi_nlp_core.utils.util import read_dict, get_ngram, tokenize, padding_punct
from vi_nlp_core.utils.dep_pattern import get_dep_dict, get_dep_symptoms_dict, get_dep_keys, get_list_of_symps
from vi_nlp_core.utils.preproces import Preprocess


class SymptomMatcher:
    def __init__(self, n_grams=4, threshold=30):

        self.max_n_gram = n_grams

        self.dep_symp_dict = self.get_extend_dep_symp_dict()
        self.dep_keys = get_dep_keys()
        self.list_of_symptoms = self.get_list_of_symptoms()

        # for fuzzy match
        self.threshold = threshold
        self.preprocessor = Preprocess()

    def map_gender_to_key(self, str):
        '''
        1. Semi/Extract matching
        2. Fuzzy mathching
        '''
        str = str.lower()
        res = self.extract_matching(str)

        if res != None:
            res = self.format_output(str, res[0], res[1])
            return res
        else:
            res = self.fuzzy_matching(str, self.threshold)
            res = self.format_output(str, res[0], res[1])
            return res

    def extract_symptoms(self, utterance, get_dep_keys=False, top_k=3):
        '''
        get_dep_keys : get_dep_keys from utterance
        top_k : use whenever get_dep_keys is True
        '''
        utterance = self.preprocess(utterance).lower()

        symptoms = self.get_symptoms(utterance)
        if symptoms != [] and get_dep_keys:
            data = []
            for ent in symptoms:
                data.append(ent['value'])
            result = self.get_department_from_symptoms(data, top_k)
        elif symptoms == []:
            result = 'No symptoms are existed or some symptoms are not updated in database'
        else:
            result = {}
            result['entities'] = symptoms

        return result

    def get_symptoms(self, utterance):
        result = []
        history = []
        for i in range(1,self.max_n_gram):
            ngrams = get_ngram(utterance, i)
            for item in ngrams:
                if len(item) > 1:
                    item = ' '.join(item)
                else:
                    item = item[0]
                symptom = self.fuzzy_matching(item, threshold=85)
                if symptom != []:
                    symptom = symptom[1]
                    start = utterance.find(item)
                    end = start + len(item)
                    obj = {
                        "start": start,
                        "end": end,
                        "entity": "symptom",
                        "value": symptom,
                        "confidence": 1.0,
                        "extractor": 'fuzzy_matching'
                    }

                    if symptom not in history:
                        result.append(obj)
                        history.append(symptom)
        return result

    def extract_matching(self, str):
        for k, v in self.gender_dict.items():
            for token in v:
                if token in str.split(' '):
                    return (self.keys[k], token)
        return None

    def fuzzy_matching(self, str, threshold):
        scores = []
        for k, v in self.dep_symp_dict.items():
            ratios = [fuzz.ratio(str, value)
                      for value in v]  # ensure both are in string
            scores.append({"key": k, "score": max(ratios),
                           'value': v[ratios.index(max(ratios))]})

        filtered_scores = [
            item for item in scores if item['score'] >= threshold]
        if filtered_scores == []:
            return filtered_scores
        sorted_filtered_scores = sorted(
            filtered_scores, key=lambda k: k['score'], reverse=True)

        return (self.dep_keys[sorted_filtered_scores[0]['key']], sorted_filtered_scores[0]['value'])

    def format_output(self, text, key, value):
        return {
            'key': key,
            'text': text,
            'value': value
        }

    def get_department_score(self, symptoms, dep_symptoms):
        cnt = 0
        for item in symptoms:
            if item.lower() in dep_symptoms:
                cnt += 1
        return cnt/len(symptoms)

    def get_department_from_symptoms(self, symptoms, top_k=3):
        '''
        symptoms : list of symptoms
        '''
        score_list = []

        for i, (dep, dep_symptoms) in enumerate(self.dep_symp_dict.items()):
            score = self.get_department_score(symptoms, dep_symptoms)
            score_list.append({
                'score': score,
                'key': dep})
        filtered_scores = sorted(
            score_list, key=lambda k: k['score'], reverse=True)

        res = [(self.dep_keys[item['key']], item['score'])
               for item in filtered_scores[:top_k]]

        return res

    def get_list_of_symptoms(self):
        data = []
        for k, v in self.dep_symp_dict.items():
            data.extend(v)
        
        li_symps = get_list_of_symps()
        data.extend(li_symps)
        data = list(set(data))
        return data

    def preprocess(self, text):
        text = text.replace('\n', '').strip()
        text = re.sub('(?<! )(?=[,!?()])|(?<=[,!?()])(?! )', r' ', text)
        return text

    def get_extend_dep_symp_dict(self):
        d = {}
        d1 = get_dep_dict()
        d2 = get_dep_symptoms_dict()
        for k, v in d1.items():
            d[k] = v
            d[k].extend(d2[k])
        return d



def get_gender_dict():
    d = {}

    d['male'] = [
        'trai',
        'nam',
        'anh',
        'chú',
        'dượng',
        'cậu',
        'ông',
        'phụ thân',
        'sư thúc',
        'sư phụ',
        'rể'
    ]

    d['female'] = [
        'gái',
        'nữ',
        'chị',  
        'cô',
        'dì',
        'thím',
        'bà',
        'mợ',
        'o',
        'phụ mẫu',
        'sư mẫu',
        'dâu'
    ]
    return d


def get_gender_keys():
    keys = {
        'male': 'GEN01',
        'female': 'GEN02'
    }
    return keys



import os

def get_dep_dict():
    d = {}

    d['bác sĩ gia đình'] = [
        'bác sĩ gia đình',
        'bs gia đình',
        'bs gđ',
        'gia đình',
        'y học gia đình',
        'bsgd',
        'family medical'
    ]
    d['da liễu'] = [
        'da liễu',
        'da',
        'thẩm mĩ',
        'thẩm mý',
    ]
    d['dinh dưỡng'] = [
        'dinh dưỡng',
        'dinh dưỡng học đường',
        'ăn uông',
        'suất ăn',
        'chế biến',
        'an toàn vệ sinh thực phẩm',
        'thực phẩm'
    ]
    d['hô hấp'] = [
        'hô hấp',
        'phổi'
    ]
    d['nội thận'] = [
        'nội thận',
        'thận'
    ]
    d['nhi'] = [
        'nhi',
        'em bé',
        'nhi đồng',
    ]
    d['nội tiết'] = [
        'nội tiết'
    ]
    d['nội tiêu hóa'] = [
        'nội tiêu hóa',
        'tiêu hóa'
    ]
    d['nội tổng quát'] = [
        'nội tổng quát'
    ]
    d['tim mạch'] = [
        'tim mạch',
        'tim'
    ]
    d['tâm thần'] = [
        'tâm thần'
    ]
    d['thần kinh'] = [
        'thần kinh'
    ]
    d['tuyến vú'] = [
        'tuyến vú',
        'vú'
    ]
    d['chẩn đoán hình ảnh'] = [
        'chẩn đoán hình ảnh',
        'chuẩn đoán hình ảnh',
        'hình ảnh',
        'xquang'
    ]
    d['chấn thương chỉnh hình'] = [
        'chấn thương chỉnh hình',
        'xương khớp',
        'xương',
        'khớp',
        'ngoại'
    ]
    d['nhiễm'] = [
        'nhiễm',
        'dịch tể',
        'virus'
    ]
    d['sản phụ khoa'] = [
        'sản phụ khoa',
        'sản phụ',
        'sản',
        'sinh đẻ'
    ]
    d['tai mũi họng'] = [
        'tai mũi họng',
        'mũi',
        'họng',
    ]

    return d


def get_dep_keys():
    keys = {
        'bác sĩ gia đình': 'SP001',
        'da liễu': 'SP002',
        'dinh dưỡng': 'SP003',
        'hô hấp': 'SP004',
        'nội thận': 'SP005',
        'nhi': 'SP006',
        'nội tiết': 'SP007',
        'nội tiêu hóa': 'SP008',
        'nội tổng quát': 'SP009',
        'tim mạch': 'SP010',
        'tâm thần': 'SP011',
        'thần kinh': 'SP012',
        'tuyến vú': 'SP013',
        'chẩn đoán hình ảnh': 'SP014',
        'chấn thương chỉnh hình': 'SP015',
        'nhiễm': 'SP016',
        'sản phụ khoa': 'SP017',
        'tai mũi họng': 'SP018',
    }
    return keys


def get_dep_symptoms_dict():
    d = {}

    d['bác sĩ gia đình'] = [
        'đau bụng', 'sốt', 'ho', 'ói', 'nôn', 'chóng mặt', 'đau ngực', 'đau khớp', 'mất'
        'ngủ', 'lo âu', 'khó thở', 'tăng huyết áp', 'giảm huyết áp', 'cao huyết áp',
        'đau họng,' 'đau khớp', 'mỏi vai gáy', 'mỏi vai', 'tiểu đường', 'mệt',
    ]
    d['da liễu'] = [
        'ngứa', 'nổi sẩn', 'da dày sừng', 'bong vảy', 'mụn nước', 'bóng nước', 'chảy dịch',
        'mụn mủ', 'loét', 'mụn trứng cá', 'ngứa da', 'ngứa âm đạo', 'vảy nến',
        'dị ứng', 'phát ban', 'nổi bóng nước', 'phù mặt', 'khó thở', 'mụn', 'nám',
        'mụn cóc', 'đỏ da', 'phát ban', 'dị ứng', 'loét', 'áp-xe', 'hoại tử ', 'nổi nốt', ]
    d['dinh dưỡng'] = ['béo', 'gầy', 'mập', 'nhẹ cân',
                       'thiếu cân', 'ốm', 'tăng cân', 'suy dinh dưỡng']
    d['hô hấp'] = ['khó thở', 'đau ngực', 'sốt', 'nôn', 'ho']
    d['nội thận'] = ['mệt', 'tiểu đêm', 'đái đêm', 'tăng huyết áp',
                     'đái tháo đường', 'tiểu đường', 'phù, ói', 'nôn', 'khó thở', 'mệt', 'lơ mơ', ]
    d['nhi'] = ['sốt', 'ho', 'ói', 'nôn', 'táo bón', 'sổ mũi', 'tiêu chảy', 'đau bụng', 'nôn ói'
                'dị ứng', 'nhiễm trùng da', 'tai nạn', 'co giật', 'thở mệt', 'ọc sữa', 'ho',
                'đau đầu', 'dị ứng', 'viêm da', 'thở khò khè', ]
    d['nội tiết'] = ['sốt', 'đường huyết cao', 'nhiễm trùng', 'tuyến giáp', 'bệnh tuyến giáp',
                     'sốt', 'đau ngực', 'khó thở', ]
    d['nội tiêu hóa'] = ['đau bụng', 'tiêu chảy', 'nôn', 'táo bón', 'đi ngoài có máu', 'đi ngoài ra máu', 'vàng da',
                         'nhức đầu', 'chóng mặt', 'ói', 'sốt cao', 'lơ mơ', 'ợ chua',
                         'đầy bụng', 'đầy hơi', 'viêm dạ dày', 'loét dạ dày', 'viêm gan', 'viêm gan',
                         'siêu vi', 'viêm gan siêu vi B', 'viêm gan siêu vi C', ]
    d['nội tổng quát'] = ['đau ngực', 'ho', 'khó thở', 'đau bụng', 'nôn', 'ói', 'đau đầu', 'sốt', 'huyết áp',
                          'tăng', 'chóng mặt', 'huyết áp cao', 'sốt cao', 'khó thở', 'rối loạn tri',
                          'giác', 'tăng huyết áp', 'rối loạn tiêu hoá', 'đau khớp', 'rối loạn tiền',
                          'đình', 'đái tháo đường', 'bệnh tuyến giáp', 'viêm phế quản', ]
    d['tim mạch'] = ['đau ngực', 'khó thở', 'hồi hộp', 'chóng mặt', 'phù', 'sốt', 'huyết áp cao',
                     'ho khan', 'phù chân', 'đau ngực', ]
    d['tâm thần'] = ['mất ngủ', 'tự sát', 'trầm cảm', 'lo âu', 'bồn chồn', 'kích động', 'tự vẫn',
                     'nóng tính', 'nóng tánh', 'tự tử ']
    d['thần kinh'] = ['đau đầu', 'chóng mặt', 'mất ngủ', 'nói khó', 'khó nói', 'yếu tay',
                      'yếu chân', 'yếu chân tay', 'yếu tay chân', 'hồi hộp', 'tai biến',  'đau đầu',
                      'chóng mặt', 'rối loạn ý thức', 'co giật', 'rối loạn ngôn ngữ','căng thẳng' ]
    d['tuyến vú'] = ['u vú', 'đau vú', 'chảy dịch núm vú', ]
    d['chẩn đoán hình ảnh'] = ['ho', 'sốt', 'khó thở', 'ho ra máu', 'đau bụng', 'ói', 'nôn', 'tiêu chảy', 'đi ngoài',
                               'táo bón', 'rối loạn tri giác', 'liệt nửa người', 'suy hô hấp', 'x-quang', ]
    d['chấn thương chỉnh hình'] = ['đau lưng', 'đau vai gáy', 'đau gối háng', 'gãy xương', 'trật khớp', 'đau lưng',
                                   'thoái hóa khớp', ]
    d['nhiễm'] = ['sốt', 'đau đầu', 'ói', 'phát ban', 'co giật', 'rối loạn tri giác', 'suy hô hấp',
                  'tiêu chảy', 'đi ngoài', 'thủy đậu', 'quai bị', 'covid-19', ]
    d['sản phụ khoa'] = ['ngứa âm đạo', 'huyết trắng', 'ngứa âm hộ', ]
    d['tai mũi họng'] = ['ho', 'sổ mũi', 'đau họng', 'đau đầu', 'nghẹt mũi', 'mất mùi', 'chảy máu mũi',
                         'khàn tiếng', 'viêm họng', 'viêm xoang', ]

    return d

ROOT_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
SYMPTOMS_PATH = os.path.join(ROOT_DIR,"data/symptoms.txt")

def get_list_of_symps():
    data = []
    with open(SYMPTOMS_PATH,'r',encoding='utf-8') as f:
        for line in f:
            data.append(line.replace('\n',''))
    return data 


import os
from setuptools import setup, find_packages

with open("requirements.txt") as f:
    reqs = f.read().splitlines()
    
print(reqs)
VERSION = os.getenv('PACKAGE_VERSION', 'v0.0.1')[1:]

setup(
    name='vi_nlp_core',
    version='1.1.9',
    description='vi-core-nlp is a library that supports Vietnamese NER by pattern matching .',
    long_description=open("README.md").read(),
    long_description_content_type='text/markdown',
    include_package_data=True,
    url='https://github.com/demdecuong/vi-core-nlp/tree/master',
    packages=find_packages(),
    author='minhnp et al.',
    author_email='nguyenphucminh2804@gmail.com',
    classifiers=[
        'Programming Language :: Python :: 3',
    ],
    install_requires=reqs,
    keywords='vi_nlp_core',
    python_requires='>=3.6',
    py_modules=['vi_nlp_core'],
)